@@ -5,7 +5,7 @@
 //! Resolve details about the PyOxidizer execution environment.
 
 use {
-    crate::project_layout::PyembedLocation,
+    crate::{project_layout::PyembedLocation, py_packaging::distribution::AppleSdkInfo},
     anyhow::{anyhow, Context, Result},
     once_cell::sync::Lazy,
     semver::Version,
@@ -24,7 +24,7 @@ use {
 const PYOXIDIZER_CRATE_VERSION: &str = env!("CARGO_PKG_VERSION");
 
 /// Version string of pyembed crate from its Cargo.toml.
-const PYEMBED_CRATE_VERSION: &str = "0.15.0";
+const PYEMBED_CRATE_VERSION: &str = "0.16.0";
 
 /// URL of Git repository we were built from.
 const GIT_REPO_URL: &str = env!("GIT_REPO_URL");
@@ -161,20 +161,19 @@ impl Default for PyOxidizerSource {
 impl PyOxidizerSource {
     /// Determine the location of the pyembed crate given a run-time environment.
     ///
-    /// If we were built from a Git clone and we have access to that path, we use
-    /// a local filesystem path.
+    /// If not a pre-release version, we always reference the plain version,
+    /// presumably available in a registry.
     ///
-    /// Otherwise, if the pyembed crate version isn't a pre-release version, we
-    /// reference the package registry. But if it is a pre-release version, we
-    /// reference a specific commit in this project's Git repository.
+    /// If it is a pre-release version, we reference a local filesystem path, if
+    /// available, or a specific commit in this project's Git repository.
     pub fn as_pyembed_location(&self) -> PyembedLocation {
-        match self {
-            PyOxidizerSource::LocalPath { path } => {
-                PyembedLocation::Path(canonicalize_path(&path.join("pyembed")).unwrap())
-            }
-            PyOxidizerSource::GitUrl { url, commit, tag } => {
-                // Pre-release version only available via Git reference.
-                if PYEMBED_CRATE_VERSION.ends_with("-pre") {
+        // Pre-release version only available via path or Git references.
+        if PYEMBED_CRATE_VERSION.ends_with("-pre") {
+            match self {
+                PyOxidizerSource::LocalPath { path } => {
+                    PyembedLocation::Path(canonicalize_path(&path.join("pyembed")).unwrap())
+                }
+                PyOxidizerSource::GitUrl { url, commit, tag } => {
                     if let Some(tag) = tag {
                         PyembedLocation::Git(url.clone(), tag.clone())
                     } else if let Some(commit) = commit {
@@ -183,10 +182,11 @@ impl PyOxidizerSource {
                         // We shouldn't get here. But who knows what's possible.
                         PyembedLocation::Git(url.clone(), "main".to_string())
                     }
-                } else {
-                    PyembedLocation::Version(PYEMBED_CRATE_VERSION.to_string())
                 }
             }
+        } else {
+            // Published version is always a plain version reference to the registry.
+            PyembedLocation::Version(PYEMBED_CRATE_VERSION.to_string())
         }
     }
 
@@ -416,6 +416,48 @@ impl Environment {
             rust_version,
         })
     }
+
+    /// Attempt to resolve an appropriate Apple SDK to use given settings.
+    pub fn resolve_apple_sdk(
+        &self,
+        logger: &slog::Logger,
+        sdk_info: &AppleSdkInfo,
+    ) -> Result<AppleSdk> {
+        let platform = &sdk_info.platform;
+        let minimum_version = &sdk_info.version;
+        let deployment_target = &sdk_info.deployment_target;
+
+        let sdk = if let Ok(sdk_root) = std::env::var("SDKROOT") {
+            warn!(logger, "SDKROOT defined; using Apple SDK at {}", &sdk_root);
+
+            AppleSdk::from_directory(&PathBuf::from(&sdk_root)).with_context(|| {
+                format!("resolving Apple SDK at {} as defined by SDKROOT", sdk_root)
+            })?
+        } else {
+            warn!(
+                logger,
+                "locating Apple SDK {}{}+ supporting {}{}",
+                platform,
+                minimum_version,
+                platform,
+                deployment_target
+            );
+
+            resolve_apple_sdk(logger, platform, minimum_version, deployment_target)
+                .context("resolving Apple SDK")?
+        };
+
+        warn!(
+            logger,
+            "using SDK {} ({} targeting {}{})",
+            sdk.path.display(),
+            sdk.name,
+            platform,
+            deployment_target
+        );
+
+        Ok(sdk)
+    }
 }
 
 /// Represents an available Rust toolchain.
