@@ -4,7 +4,7 @@
 
 use {
     crate::{
-        environment::{canonicalize_path, resolve_apple_sdk, Environment, RustEnvironment},
+        environment::{canonicalize_path, Environment, RustEnvironment},
         project_layout::initialize_project,
         py_packaging::{
             binary::{EmbeddedPythonContext, LibpythonLinkMode, PythonBinaryBuilder},
@@ -23,7 +23,6 @@ use {
         io::{BufRead, BufReader},
         path::{Path, PathBuf},
     },
-    tugger_apple::AppleSdk,
 };
 
 pub const HOST: &str = env!("HOST");
@@ -169,43 +168,14 @@ impl BuildEnvironment {
                 anyhow!("targeting Apple platform but Apple SDK info not available")
             })?;
 
-            let platform = &sdk_info.platform;
-            let minimum_version = &sdk_info.version;
-            let deployment_target = &sdk_info.deployment_target;
+            let sdk = env
+                .resolve_apple_sdk(logger, sdk_info)
+                .context("resolving Apple SDK")?;
 
-            // Respect the SDKROOT environment variable.
-            let sdk = if let Some(sdk_root) = envs.get("SDKROOT") {
-                warn!(logger, "SDKROOT defined; using Apple SDK at {}", sdk_root);
-                AppleSdk::from_directory(&PathBuf::from(sdk_root)).with_context(|| {
-                    format!("resolving SDK at {} as defined via SDKROOT", sdk_root)
-                })?
-            } else {
-                warn!(
-                    logger,
-                    "locating Apple SDK {}{}+ supporting {}{}",
-                    platform,
-                    minimum_version,
-                    platform,
-                    deployment_target
-                );
-
-                resolve_apple_sdk(logger, platform, minimum_version, deployment_target)
-                    .context("resolving Apple SDK")?
-            };
-
-            warn!(
-                logger,
-                "using SDK {} ({} targeting {}{})",
-                sdk.path.display(),
-                sdk.name,
-                platform,
-                deployment_target
-            );
-
-            let deployment_target_name = sdk.supported_targets.get(platform).ok_or_else(|| {
-                anyhow!("could not find settings for target {} (this shouldn't happen)", platform)
+            let deployment_target_name = sdk.supported_targets.get(&sdk_info.platform).ok_or_else(|| {
+                anyhow!("could not find settings for target {} (this shouldn't happen)", &sdk_info.platform)
             })?.deployment_target_setting_name.as_ref().ok_or_else(|| {
-                anyhow!("unable to identify deployment target environment variable for {} (please report this bug)", platform)
+                anyhow!("unable to identify deployment target environment variable for {} (please report this bug)", &sdk_info.platform)
             })?;
 
             // SDKROOT will instruct rustc and potentially other tools to use exactly this SDK.
@@ -217,7 +187,7 @@ impl BuildEnvironment {
             if envs.get(deployment_target_name).is_none() {
                 envs.insert(
                     deployment_target_name.to_string(),
-                    deployment_target.to_string(),
+                    sdk_info.deployment_target.clone(),
                 );
             }
         }
@@ -257,6 +227,13 @@ impl BuildEnvironment {
             );
         }
 
+        // We want cargo to use the rustc from our resolved Rust environment. So
+        // always set RUSTC to force it.
+        envs.insert(
+            "RUSTC".to_string(),
+            format!("{}", rust_environment.rustc_exe.display()),
+        );
+
         Ok(Self {
             rust_environment,
             environment_vars: envs,
@@ -300,7 +277,7 @@ pub fn build_executable_with_rust_project<'a>(
 
     // Derive and write the artifacts needed to build a binary embedding Python.
     let embedded_data = exe
-        .to_embedded_python_context(logger, opt_level)
+        .to_embedded_python_context(logger, env, opt_level)
         .context("obtaining embedded python context")?;
     embedded_data
         .write_files(&artifacts_path)
@@ -387,11 +364,12 @@ pub fn build_executable_with_rust_project<'a>(
     {
         let reader = BufReader::new(&command);
         for line in reader.lines() {
-            warn!(logger, "{}", line?);
+            warn!(logger, "{}", line.context("reading cargo output")?);
         }
     }
     let output = command
-        .try_wait()?
+        .try_wait()
+        .context("waiting on cargo process")?
         .ok_or_else(|| anyhow!("unable to wait on command"))?;
     if !output.status.success() {
         return Err(anyhow!("cargo build failed"));
@@ -409,7 +387,8 @@ pub fn build_executable_with_rust_project<'a>(
         return Err(anyhow!("{} does not exist", exe_path.display()));
     }
 
-    let exe_data = std::fs::read(&exe_path)?;
+    let exe_data =
+        std::fs::read(&exe_path).with_context(|| format!("reading {}", exe_path.display()))?;
     let exe_name = exe_path.file_name().unwrap().to_string_lossy().to_string();
 
     Ok(BuiltExecutable {
