@@ -527,6 +527,14 @@ impl StandaloneDistribution {
                     let mut entry =
                         entry.map_err(|e| anyhow!("failed to iterate over archive: {}", e))?;
 
+                    // The mtimes in the archive may be 0 / UNIX epoch. This shouldn't
+                    // matter. However, pip will sometimes attempt to produce a zip file of
+                    // its own content and Python's zip code won't handle times before 1980,
+                    // which is later than UNIX epoch. This can lead to pip blowing up at
+                    // run-time. We work around this by not adjusting the mtime when
+                    // extracting the archive. This effectively makes the mtime "now."
+                    entry.set_preserve_mtime(false);
+
                     // Windows doesn't support symlinks without special permissions.
                     // So we track symlinks explicitly and copy files post extract if
                     // running on that platform.
